VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "Admin"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Const MODULE_NAME As String = "Bank_DB.Bank"

Private m_objDBHelper As DBHelper

Private Const BANK_DESC_SIZE = 20
Private Const BANK_DSN_SIZE = 250
Private Const CARD_PASSWORD_SIZE = 10

Private Sub Class_Initialize()
' Initialize the helper object to run stored procedures
   Set m_objDBHelper = CreateObject("Bank_DB.DBHelper")
End Sub

Private Sub Class_Terminate()
' Destroy the helper object
   Set m_objDBHelper = Nothing
End Sub

Public Property Get GetAccountType(ByVal BankID As Integer, _
  ByVal CardId As Long) As ADODB.Recordset

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Return a recordset of all the
  ' account types that are listed for this CardID
  On Error Resume Next
  Set GetAccountType = m_objDBHelper.RunSPReturnRS( _
    strDSN, "sp_select_AccountType_By_CardId", _
    Array("@CardId_1", adInteger, adSizeInteger, CardId))
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "GetAccountType"
  GoTo Clean_Up
End Property

Public Property Get GetAccountTypes(ByVal BankID As Integer) _
  As ADODB.Recordset

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Return a recordset of all the
  ' account types and corresponding AccountID.
  On Error Resume Next
  Set GetAccountTypes = m_objDBHelper.RunSPReturnRS( _
    strDSN, "sp_select_AccountType")
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "GetAccountTypes"
  GoTo Clean_Up
End Property

Public Property Get AddBank(ByVal BankID As Integer, _
  ByVal NewBankId As Integer, ByVal Description As String, _
  DSN As String) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim lngReturn As Long
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Insert a new bank in the table.
  On Error Resume Next
  lngReturn = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_insert_Bank", _
    Array("@BankId_1", adSmallInt, adSizeSmallInt, NewBankId), _
    Array("@BankDesc_2", adChar, BANK_DESC_SIZE, Description), _
    Array("@BankDSN_3", adVarChar, BANK_DSN_SIZE, DSN))
  
  If lngReturn = 0 Then
    AddBank = True
  Else
    AddBank = False
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "AddBank"
  GoTo Clean_Up
End Property

Public Property Get EditBank(ByVal BankID As Integer, _
  ByVal ChangeBankId As Integer, ByVal Description As String, _
  DSN As String) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim lngReturn As Long
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Update the bank record.
  On Error Resume Next
  lngReturn = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_update_Bank", _
    Array("@BankId_1", adSmallInt, adSizeSmallInt, ChangeBankId), _
    Array("@BankDesc_2", adChar, BANK_DESC_SIZE, Description), _
    Array("@BankDSN_3", adVarChar, BANK_DSN_SIZE, DSN))
  
  If lngReturn = 0 Then
    EditBank = True
  Else
    EditBank = False
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "EditBank"
  GoTo Clean_Up
End Property

Public Property Get DeleteBank(ByVal BankID As Integer, _
  ByVal ChangeBankId As Integer) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim lngReturn As Long
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Update the bank record.
  On Error Resume Next
  lngReturn = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_delete_Bank", _
    Array("@BankId_1", adSmallInt, adSizeSmallInt, ChangeBankId))
  
  If lngReturn = 0 Then
    DeleteBank = True
  Else
    DeleteBank = False
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "DeleteBank"
  GoTo Clean_Up
End Property

Public Property Get VerifyCard(ByVal BankID As Integer, _
  ByVal CardId As Long) As Long
' Verify that the selected card account exists at the selected bank

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  If strDSN <> "" Then
    ' If the bank exists run the stored procedure.
    ' Returns the count of accounts that have this AccountID
    VerifyCard = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_verify_CardAccount_By_CardId", _
      Array("@CardId_1", adInteger, adSizeInteger, _
      CardId))
    CtxSetComplete
  Else
    VerifyCard = 0 'Set to no accounts
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "VerifyCard"
  VerifyCard = 0 'Set to no accounts
  GoTo Clean_Up
End Property

Public Property Get DeleteCard(ByVal BankID As Integer, _
  ByVal CardId As Long) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim recAccountID As ADODB.Recordset
  Dim lngAccountID As Long
  Dim lngReturn As Long
  Dim strDSN As String
  
  On Error GoTo ErrorHandler
  DeleteCard = False
  
  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Delete all the related records.
  ' Leave the AccountHistory records as an audit trail.
  On Error Resume Next
  Set recAccountID = m_objDBHelper.RunSPReturnRS( _
    strDSN, "sp_select_AccountId_By_CardId", _
    Array("@CardId_1", adInteger, adSizeInteger, CardId))
  
  Do While Not recAccountID.EOF
    lngAccountID = recAccountID!AccountID
    
    m_objDBHelper.RunSP strDSN, "sp_delete_CardxAccount", _
      Array("@AccountId_1", adInteger, adSizeInteger, lngAccountID)
    
    recAccountID.MoveNext
  Loop
  
  recAccountID.MoveFirst
  
  Do While Not recAccountID.EOF
    lngAccountID = recAccountID!AccountID
    
    m_objDBHelper.RunSP strDSN, "sp_delete_BankAccount", _
      Array("@AccountId_1", adInteger, adSizeInteger, lngAccountID)
    
    recAccountID.MoveNext
  Loop
  
  m_objDBHelper.RunSP strDSN, "sp_delete_WROBACard", _
    Array("@CardId_1", adInteger, adSizeInteger, CardId)
    
  DeleteCard = True
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "DeleteCard"
  GoTo Clean_Up
End Property

Public Property Get AddCard(ByVal BankID As Integer, _
  ByVal AccountType As Integer, ByVal InitialBalance As Currency, _
  ByVal DailyLimit As Currency, ByVal OverDraft As Currency, _
  ByVal Password As String, ByVal CardId As Long, _
  ByVal AccountID As Long) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim lngReturn As Long
  Dim strDSN As String
  
  On Error GoTo ErrorHandler
  AddCard = False

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Insert a new card in the
  ' related tables.
  On Error Resume Next
  lngReturn = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_insert_WROBACard", _
    Array("@CardId_1", adInteger, adSizeInteger, CardId), _
    Array("@BankDesc_2", adChar, CARD_PASSWORD_SIZE, Password), _
    Array("@InvalidPasswordCnt_3", adTinyInt, adSizetinyInt, 0))
  
  If lngReturn = 0 Then
    lngReturn = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_insert_BankAccount", _
      Array("@AccountId_1", adInteger, adSizeInteger, AccountID), _
      Array("@AccountType_2", adTinyInt, adSizetinyInt, AccountType), _
      Array("@Balance_3", adCurrency, adSizeCurrency, InitialBalance), _
      Array("@Limit_4", adCurrency, adSizeCurrency, DailyLimit), _
      Array("@Remaining_5", adCurrency, adSizeCurrency, DailyLimit), _
      Array("@LastDate_6", adDBTimeStamp, adSizeDateTime, Now), _
      Array("@Overdraft_7", adCurrency, adSizeCurrency, OverDraft))
  End If
  
  If lngReturn = 0 Then
    lngReturn = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_insert_CardxAccount", _
      Array("@Card_1", adInteger, adSizeInteger, CardId), _
      Array("@AccountType_2", adTinyInt, adSizetinyInt, AccountType), _
      Array("@AccountID_3", adInteger, adSizeInteger, AccountID))
  End If
  
  If lngReturn = 0 Then
    AddCard = True
  Else
    AddCard = False
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "AddCard"
  GoTo Clean_Up
End Property

Public Property Get GetLastCardID(ByVal BankID As Integer) As Long
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Return a recordset of all the
  ' account types that are listed for this CardID
  On Error Resume Next
  GetLastCardID = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_select_Last_CardId")
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "GetLastCardID"
  GoTo Clean_Up
End Property

Public Property Get GetLastAccountId(ByVal BankID As Integer) As Long
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Return a recordset of all the
  ' account types that are listed for this CardID
  On Error Resume Next
  GetLastAccountId = m_objDBHelper.RunSPReturnInteger( _
    strDSN, "sp_select_Last_AccountId")
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "GetLastAccountId"
  GoTo Clean_Up
End Property
