VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "Payees"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in data tier

Const MODULE_NAME As String = "Bank_DB.Payees"

' Size for strings in table
Const PAYEEDESC_SIZE As Integer = 30
Const PAYEEID_SIZE   As Integer = 36

Private m_objDBHelper As DBHelper

Private Const UPDSUCCESS = 0
Private Const UPDERROR = 1

Private Sub Class_Initialize()
' Initialize the helper object to run stored procedures
   Set m_objDBHelper = CreateObject("Bank_DB.DBHelper")
End Sub

Private Sub Class_Terminate()
' Destroy the helper object
   Set m_objDBHelper = Nothing
End Sub

Public Property Get MatchPayee(ByVal BankID As Integer, _
  ByVal CardID As Long) As Boolean

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  Dim lngPayees As Long
  
  On Error GoTo ErrorHandler
  MatchPayee = False
  
  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
 
  If strDSN <> "" Then
    lngPayees = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_verify_Payee_By_CardId", _
      Array("@CardId_1", adInteger, adSizeInteger, _
      CardID))
    If lngPayees = 0 Then
      MatchPayee = True
    End If
  End If
    
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "MatchPayee"
  GoTo Clean_Up
End Property

Public Property Get InsertPayees(ByVal PayeeDescription As String, _
  ByVal AccountID As Long, ByVal DefaultAmount As Currency, _
  ByVal BankID As Integer, ByVal CardID As Long, _
  ByVal WROBACardID As Long) As Long

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  If strDSN <> "" Then
    On Error Resume Next
    ' Run the stored procedure
    InsertPayees = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_insert_Payee", _
      Array("@CardId_2", adInteger, adSizeInteger, CardID), _
      Array("@PayeeDesc_3", adChar, PAYEEDESC_SIZE, _
        PayeeDescription), _
      Array("@BankId_4", adSmallInt, adSizeSmallInt, BankID), _
      Array("@AccountId_5", adInteger, adSizeInteger, AccountID), _
      Array("@DefaultAmount_6", adCurrency, adSizeCurrency, _
        DefaultAmount), _
      Array("@WROBACardId_7", adInteger, adSizeInteger, WROBACardID))
  
    If InsertPayees > 0 Then InsertPayees = UPDERROR
  Else
    InsertPayees = UPDERROR
  End If
    
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "InsertPayees"
  InsertPayees = UPDERROR
  GoTo Clean_Up
End Property

Public Property Get EditPayee(ByVal PayeeID As String, _
  ByVal PayeeDescription As String, ByVal DefaultAmount As Currency, _
  ByVal BankID As Integer) As Long

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)

  If strDSN <> "" Then
    ' Run the stored procedure
    EditPayee = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_update_payee", _
      Array("@PayeeId_1", adChar, PAYEEID_SIZE, PayeeID), _
      Array("@PayeeDesc_2", adChar, PAYEEDESC_SIZE, _
        PayeeDescription), _
      Array("@DefaultAmount_3", adCurrency, adSizeCurrency, _
        DefaultAmount))
  Else
    EditPayee = UPDERROR
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "EditPayee"
  EditPayee = UPDERROR
  GoTo Clean_Up
End Property

Public Property Get DeletePayee(ByVal PayeeID As String, _
  ByVal BankID As Integer, ByVal PayeeDescription As String) _
  As Long

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  If strDSN <> "" Then
    ' Run the stored procedure
    DeletePayee = m_objDBHelper.RunSPReturnInteger( _
      strDSN, "sp_delete_payee", _
      Array("@PayeeId_1", adChar, PAYEEID_SIZE, PayeeID))
  Else
    DeletePayee = UPDERROR
  End If
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "DeletePayee"
  DeletePayee = UPDERROR
  GoTo Clean_Up
End Property

Public Property Get GetAllPayees(ByVal BankID As Integer, _
  ByVal CardID As Long) As ADODB.Recordset
   
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler
  Set GetAllPayees = Nothing
  
  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure. Return a recordset of all the payees
  ' that are in the table for this WROBA card
  On Error Resume Next
  Set GetAllPayees = m_objDBHelper.RunSPReturnRS( _
    strDSN, "sp_select_All_Payees", _
    Array("@CardId_1", adInteger, adSizeInteger, CardID))
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetAllPayees"
  GoTo Clean_Up
End Property

