VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Customer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"374879F40173"
'This is the UI-centric stateful Customer class.  It is responsible to create
'and release the data centric stateless CustomerN (non-transact) and CustomerT
'(transact customer) objects.

Option Explicit

Public Event Changed()

Private Const mcClassName As String = "UIObjects.Customer"
Private Const mcObjectError As String = vbObjectError + &H1100

'Customer ID
Private mID As String

'Customer Name
Private mName As String

Private mIsNew As Boolean

Private mCustomerN As DCCustomer.CustomerN

Private mCustomerT As DCCustomer.CustomerT

'ADDED BY CAB
Private mrecCustomers As ADODB.Recordset


'Retrieve a customer record and populate the object properties.  If an ID is
'given, it reads a customer record by ID, if an ID is not given, it reads a
'customer record by Name.
Public Sub Read(Optional ID As String = "")
    On Error GoTo ErrHandler
    
    Dim rs As ADODB.Recordset
    Set rs = mCustomerN.GetByID(ID)
    Populate rs
    
    'ADDED BY CAB
    If mrecCustomers Is Nothing Then
      Set mrecCustomers = rs.Clone(adLockReadOnly)
    End If

FuncExit:
    On Error Resume Next
    If Not rs Is Nothing Then
        If rs.State = adStateOpen Then
            rs.Close
        End If
        Set rs = Nothing
    End If
    Exit Sub
    
ErrHandler:
    Err.Raise mcObjectError + 1, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Save a customer record.
Public Sub Save()
    On Error GoTo ErrHandler
    
    Dim CustomerData As String
    CustomerData = Serialize
    mCustomerT.Save CustomerData
    IsNew = False
    
FuncExit:
    Exit Sub
    
ErrHandler:
    Err.Raise mcObjectError + 2, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Delete a customer record.
Public Sub Delete()
    On Error GoTo ErrHandler
    mCustomerT.Delete ID
    Populate
    
FuncExit:
    Exit Sub
    
ErrHandler:
    Err.Raise mcObjectError + 3, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Populate object properties from an ADODB.Recordset retrieved by the Read method.
Private Sub Populate(Optional CustomerRS As ADODB.Recordset = Nothing)
    On Error GoTo ErrHandler
    
    If Not CustomerRS Is Nothing Then
        With CustomerRS
            If Not (.BOF And .EOF) Then
                ID = .Fields("CustomerID").Value
                Name = .Fields("CompanyName").Value
            End If
        End With
    Else
        ID = ""
        Name = ""
    End If
    
    RaiseEvent Changed
    
FuncExit:
    Exit Sub
    
ErrHandler:
    Err.Raise mcObjectError + 11, mcClassName, Err.Description
    Resume FuncExit
End Sub

Public Sub Clear()
    On Error Resume Next
    Populate
End Sub

'Serialize the Customer data.  The resulting string is send by the Save method.
Private Function Serialize() As String
    On Error GoTo ErrHandler
    
    Dim PropBag As PropertyBag
    Set PropBag = New PropertyBag
    With PropBag
        .WriteProperty "IsNew", IsNew
        .WriteProperty "ID", ID
        .WriteProperty "Name", Name
        Serialize = .Contents
    End With
    
FuncExit:
    On Error Resume Next
    If Not PropBag Is Nothing Then
        Set PropBag = Nothing
    End If
    
    Exit Function
    
ErrHandler:
    Err.Raise mcObjectError + 12, mcClassName, Err.Description
    Resume FuncExit
End Function

'Returns the Customer's name.
Public Property Get Name() As String
   Let Name = mName
End Property

'Assign a name to a customer.
Public Property Let Name(ByVal Value As String)
    Let mName = Value
End Property

'Returns a Customer ID.
Public Property Get ID() As String
   Let ID = mID
End Property

Public Property Let ID(ByVal Value As String)
    Let mID = Value
End Property

Public Property Get IsNew() As Boolean
    Let IsNew = mIsNew
End Property

Public Property Let IsNew(ByVal Value As Boolean)
    Let mIsNew = Value
    If Value Then
        Populate
    End If
End Property

Private Sub Class_Initialize()
    On Error GoTo ErrHandler
    
    Set mCustomerN = CreateObject("DCCustomer.CustomerN")
    Set mCustomerT = CreateObject("DCCustomer.CustomerT")
    IsNew = False
    
FuncExit:
    Exit Sub
    
ErrHandler:
    Err.Raise mcObjectError, mcClassName, Err.Description
    Resume FuncExit
End Sub

Private Sub Class_Terminate()
    On Error Resume Next
    Set mCustomerN = Nothing
    Set mCustomerT = Nothing
End Sub

'ADDED BY CAB
Public Property Get CustomerList() As ADODB.Recordset

  Set CustomerList = mrecCustomers

End Property

'ADDED BY CAB
Public Property Set CustomerList(vdata As ADODB.Recordset)

  Set mrecCustomers = vdata

End Property

